import dayjs from "dayjs";
import * as React from "react";
import { ViewStyle } from "react-native";

import { MIN_HEIGHT } from "./styles/commonStyles";
import {
    DateRangeHandler,
    EventCellStyle,
    EventRenderer,
    HeaderRenderer,
    HorizontalDirection,
    ICalendarEvent,
    Mode,
    MonthHeaderRenderer,
    WeekNum,
} from "./interfaces";
import { useTheme } from "./theme/ThemeContext";
import {
    getDatesInMonth,
    getDatesInNextCustomDays,
    getDatesInNextOneDay,
    getDatesInNextThreeDays,
    getDatesInWeek,
    isAllDayEvent,
    modeToNum,
    typedMemo,
} from "./utils";
import { CalendarBody } from "./CalendarBody";
import { CalendarBodyForMonthView } from "./CalendarBodyForMonthView";
import { CalendarHeader } from "./CalendarHeader";
import { CalendarHeaderForMonthView } from "./CalendarHeaderForMonthView";

export interface CalendarContainerProps<T> {
    /**
     * Events to be rendered. This is a required prop.
     */
    events: ICalendarEvent<T>[]

    /**
     * The height of calendar component. This is a required prop.
     */
    height: number

    /**
     * Adjusts the indentation of events that occur during the same time period. Defaults to 20 on web and 8 on mobile.
     */
    overlapOffset?: number

    // Custom style
    eventCellStyle?: EventCellStyle<T>
    calendarContainerStyle?: ViewStyle
    headerContainerStyle?: ViewStyle
    bodyContainerStyle?: ViewStyle

    // Custom renderer
    renderEvent?: EventRenderer<T>
    renderHeader?: HeaderRenderer<T>
    renderHeaderForMonthView?: MonthHeaderRenderer

    ampm?: boolean
    date?: Date
    locale?: string
    hideNowIndicator?: boolean
    mode?: Mode
    scrollOffsetMinutes?: number
    showTime?: boolean

    swipeEnabled?: boolean
    weekStartsOn?: WeekNum
    onChangeDate?: DateRangeHandler
    onPressCell?: (date: Date) => void
    onPressDateHeader?: (date: Date) => void
    onPressEvent?: (event: ICalendarEvent<T>) => void
    weekEndsOn?: WeekNum
    maxVisibleEventCount?: number
}

function _CalendarContainer<T>({
    events,
    height,
    ampm = false,
    date,
    eventCellStyle,
    locale = "en",
    hideNowIndicator = false,
    mode = "week",
    overlapOffset,
    scrollOffsetMinutes = 0,
    showTime = true,
    headerContainerStyle = {},
    bodyContainerStyle = {},
    swipeEnabled = true,
    weekStartsOn = 0,
    onChangeDate,
    onPressCell,
    onPressDateHeader,
    onPressEvent,
    renderEvent,
    renderHeader: HeaderComponent = CalendarHeader,
    renderHeaderForMonthView: HeaderComponentForMonthView = CalendarHeaderForMonthView,
    weekEndsOn = 6,
    maxVisibleEventCount = 3,
}: CalendarContainerProps<T>) {
    const [targetDate, setTargetDate] = React.useState(dayjs(date));

    React.useEffect(() => {
        if (date) {
            setTargetDate(dayjs(date));
        }
    }, [date]);

    const allDayEvents = React.useMemo(
        () => events.filter((event) => isAllDayEvent(event.start, event.end)),
        [events],
    );

    const daytimeEvents = React.useMemo(
        () => events.filter((event) => !isAllDayEvent(event.start, event.end)),
        [events],
    );

    const dateRange = React.useMemo(() => {
        switch (mode) {
        case "month":
            return getDatesInMonth(targetDate, locale);
        case "week":
            return getDatesInWeek(targetDate, weekStartsOn, locale);
        case "3days":
            return getDatesInNextThreeDays(targetDate, locale);
        case "day":
            return getDatesInNextOneDay(targetDate, locale);
        case "custom":
            return getDatesInNextCustomDays(targetDate, weekStartsOn, weekEndsOn, locale);
        default:
            throw new Error(
                `[react-native-big-calendar] The mode which you specified "${mode}" is not supported.`,
            );
        }
    }, [mode, targetDate, locale, weekEndsOn, weekStartsOn]);

    React.useEffect(() => {
        if (onChangeDate) {
            onChangeDate([dateRange[0].toDate(), dateRange.slice(-1)[0].toDate()]);
        }
    }, [dateRange, onChangeDate]);

    const cellHeight = React.useMemo(() => Math.max(height - 30, MIN_HEIGHT) / 24, [height]);

    const theme = useTheme();

    const onSwipeHorizontal = React.useCallback(
        (direction: HorizontalDirection) => {
            if (!swipeEnabled) {
                return;
            }
            if ((direction === "LEFT" && !theme.isRTL) || (direction === "RIGHT" && theme.isRTL)) {
                setTargetDate(targetDate.add(modeToNum(mode, targetDate), "day"));
            } else {
                setTargetDate(targetDate.add(modeToNum(mode, targetDate) * -1, "day"));
            }
        },
        [swipeEnabled, targetDate, mode, theme.isRTL],
    );

    const commonProps = {
        cellHeight,
        dateRange,
        mode,
    };

    if (mode === "month") {
        const headerProps = {
            style: headerContainerStyle,
            locale: locale,
            weekStartsOn: weekStartsOn,
        };
        return (
            <React.Fragment>
                <HeaderComponentForMonthView {...headerProps} />
                <CalendarBodyForMonthView<T>
                    {...commonProps}
                    style={bodyContainerStyle}
                    containerHeight={height}
                    events={daytimeEvents}
                    eventCellStyle={eventCellStyle}
                    weekStartsOn={weekStartsOn}
                    hideNowIndicator={hideNowIndicator}
                    onPressCell={onPressCell}
                    onPressEvent={onPressEvent}
                    onSwipeHorizontal={onSwipeHorizontal}
                    renderEvent={renderEvent}
                    targetDate={targetDate}
                    maxVisibleEventCount={maxVisibleEventCount}
                />
            </React.Fragment>
        );
    }

    const headerProps = {
        ...commonProps,
        style: headerContainerStyle,
        allDayEvents: allDayEvents,
        onPressDateHeader: onPressDateHeader,
    };

    return (
        <React.Fragment>
            <HeaderComponent {...headerProps} />
            <CalendarBody
                {...commonProps}
                style={bodyContainerStyle}
                containerHeight={height}
                events={daytimeEvents}
                eventCellStyle={eventCellStyle}
                hideNowIndicator={hideNowIndicator}
                overlapOffset={overlapOffset}
                scrollOffsetMinutes={scrollOffsetMinutes}
                ampm={ampm}
                showTime={showTime}
                onPressCell={onPressCell}
                onPressEvent={onPressEvent}
                onSwipeHorizontal={onSwipeHorizontal}
                renderEvent={renderEvent}
            />
        </React.Fragment>
    );
}

export const CalendarContainer = typedMemo(_CalendarContainer);
